@@ -689,6 +689,8 @@ pcfont_do_reset(pcl_state_t * pcs, pcl_reset_type_t type)
 
     if ((type & pcl_reset_initial) != 0) {
         pcs->font_dir = gs_font_dir_alloc(pcs->memory);
+        if (pcs->font_dir == NULL)
+            return_error(gs_error_VMerror);
         if (pcs->nocache)
             gs_setcachelimit(pcs->font_dir, 0);
         /* disable hinting at high res */
