@@ -183,7 +183,7 @@ pl_main_init_with_args(pl_main_instance_t *inst, int argc, char *argv[])
     /* debug flags we reset this out of gs_lib_init0 which sets these
        and the allocator we want the debug setting but we do our own
        allocator */
-#ifdef PACIFY_VALGRIND
+#if defined(PACIFY_VALGRIND) && defined VALGRIND_HG_DISABLE_CHECKING
     VALGRIND_HG_DISABLE_CHECKING(gs_debug, 128);
 #endif
     memset(gs_debug, 0, 128);
@@ -208,8 +208,10 @@ pl_main_init_with_args(pl_main_instance_t *inst, int argc, char *argv[])
     gs_c_param_list_write(&inst->params, mem);
     gs_param_list_set_persistent_keys((gs_param_list *)&inst->params, false);
 
-#if defined(__WIN32__) && !defined(GS_NO_UTF8)
+#if defined(__WIN32__) && defined(GS_NO_UTF8)
     arg_get_codepoint = gp_local_arg_encoding_get_codepoint;
+#else
+    arg_get_codepoint = NULL;
 #endif
     arg_init(&inst->args, (const char **)argv, argc, pl_main_arg_fopen, NULL,
              arg_get_codepoint, mem);
@@ -743,7 +745,6 @@ static int check_for_special_str(pl_main_instance_t * pmi, const char *arg, gs_p
     return 1;
 }
 
-#define arg_heap_copy(str) arg_copy(str, pmi->memory)
 static int
 pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         pl_interp_implementation_t * pjli)
@@ -773,7 +774,7 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
 
                     /* open the device if not yet open */
                     if (pdev->is_open == 0 && (code = gs_opendevice(pdev)) < 0) {
-                        return code;
+                        break;
                     }
                     if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
                         errprintf(pmi->memory, "   --saved-pages not supported by the '%s' device.\n",
@@ -793,8 +794,6 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                             color = dev_proc(pdev, map_cmyk_color)(pdev, cmyk_white);
                         code = dev_proc(pdev, fill_rectangle)(pdev, 0, 0, pdev->width, pdev->height, color);
                     }
-                    if (code < 0)
-                        return code;
                     break;
 #endif /* not defined OMIT_SAVED_PAGES */
                 /* The following code is only to allow regression testing of saved-pages */
@@ -804,7 +803,7 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     gx_device *pdev = pmi->device;
 
                     if ((code = gs_opendevice(pdev)) < 0)
-                        return code;
+                        break;
 
                     if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
                         errprintf(pmi->memory, "   --saved-pages-test not supported by the '%s' device.\n",
@@ -824,16 +823,15 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                             color = dev_proc(pdev, map_cmyk_color)(pdev, cmyk_white);
                         code = dev_proc(pdev, fill_rectangle)(pdev, 0, 0, pdev->width, pdev->height, color);
                     }
-                    if (code < 0)
-                        return code;
-                    pmi->saved_pages_test_mode = true;
+                    if (code >= 0)
+                        pmi->saved_pages_test_mode = true;
 #endif /* OMIT_SAVED_PAGES_TEST */
                     break;
                 }
                 /* FALLTHROUGH */
             default:
                 dmprintf1(pmi->memory, "Unrecognized switch: %s\n", arg);
-                return -1;
+                code = -1;
             case '\0':
                 /* read from stdin - must be last arg */
                 continue;
@@ -853,11 +851,12 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     else {
                         /* -dDefaultBooleanIs_TRUE */
                         code = check_for_special_int(pmi, arg, (int)bval);
+                        if (code < 0) code = 0;
                         if (code == 1)
                             code =
                                 param_write_bool((gs_param_list *) params,
-                                                 arg_heap_copy(arg), &bval);
-                        continue;
+                                                 arg, &bval);
+                        break;
                     }
 
                     if (value && value[0] == '/') {
@@ -869,8 +868,8 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         code = check_for_special_str(pmi, arg, &str);
                         if (code == 1)
                             code = param_write_name((gs_param_list *) params,
-                                                    arg_heap_copy(buffer), &str);
-                        continue;
+                                                    buffer, &str);
+                        break;
                     }
                     /* Search for a non-decimal 'radix' number */
                     else if (strchr(value, '#')) {
@@ -906,10 +905,11 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         strncpy(buffer, arg, eqp - arg);
                         buffer[eqp - arg] = '\0';
                         code = check_for_special_int(pmi, arg, number);
+                        if (code < 0) code = 0;
                         if (code == 1)
                             code =
                                 param_write_int((gs_param_list *) params,
-                                                arg_heap_copy(buffer), &number);
+                                                buffer, &number);
                     } else if ((!strchr(value, '.')) &&
                                /* search for an int (no decimal), if fail try a float */
                                (sscanf(value, "%d", &vi) == 1)) {
@@ -938,10 +938,11 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         strncpy(buffer, arg, eqp - arg);
                         buffer[eqp - arg] = '\0';
                         code = check_for_special_int(pmi, arg, vi);
+                        if (code < 0) code = 0;
                         if (code == 1)
                             code =
                                 param_write_int((gs_param_list *) params,
-                                                arg_heap_copy(buffer), &vi);
+                                                buffer, &vi);
                     } else if (sscanf(value, "%f", &vf) == 1) {
                         /* create a null terminated string.  NB duplicated code. */
                         strncpy(buffer, arg, eqp - arg);
@@ -950,25 +951,27 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                         if (code == 1)
                             code =
                                 param_write_float((gs_param_list *) params,
-                                                  arg_heap_copy(buffer), &vf);
+                                                  buffer, &vf);
                     } else if (!strcmp(value, "true")) {
                         /* bval = true; */
                         strncpy(buffer, arg, eqp - arg);
                         buffer[eqp - arg] = '\0';
                         code = check_for_special_int(pmi, arg, (int)bval);
+                        if (code < 0) code = 0;
                         if (code == 1)
                             code =
                                 param_write_bool((gs_param_list *) params,
-                                                 arg_heap_copy(buffer), &bval);
+                                                 buffer, &bval);
                     } else if (!strcmp(value, "false")) {
                         bval = false;
                         strncpy(buffer, arg, eqp - arg);
                         buffer[eqp - arg] = '\0';
                         code = check_for_special_int(pmi, arg, (int)bval);
+                        if (code < 0) code = 0;
                         if (code == 1)
                             code =
                                 param_write_bool((gs_param_list *) params,
-                                                 arg_heap_copy(buffer), &bval);
+                                                 buffer, &bval);
                     } else {
                         dmprintf(pmi->memory,
                                  "Usage for -d is -d<option>=[<integer>|<float>|true|false]\n");
@@ -998,7 +1001,8 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     code =
                         param_write_int_array((gs_param_list *) params,
                                               "HWSize", &ia);
-                    pmi->page_set_on_command_line = true;
+                    if (code >= 0)
+                        pmi->page_set_on_command_line = true;
                 }
                 break;
             case 'H':
@@ -1083,7 +1087,7 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     /* we expect the language to exit properly otherwise
                        there was some sort of problem */
                     if (code != e_ExitLanguage)
-                        return (code == 0 ? -1 : code);
+                        if (code == 0) code = -1;
                     else
                         code = 0;
                 }
@@ -1098,9 +1102,10 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     if (sscanf(arg, "%d", &maxk) != 1) {
                         dmprintf(pmi->memory,
                                  "-K must be followed by a number\n");
-                        return -1;
+                        code = -1;
                     }
-                    rawheap->limit = (long)maxk << 10;
+                    else
+                        rawheap->limit = (long)maxk << 10;
                 }
                 break;
             case 'o':
@@ -1201,34 +1206,32 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                     }
                     value = eqp + 1;
                     if (!strncmp(arg, "DEVICE", 6)) {
-                        int code = pl_top_create_device(pmi,
+                        code = pl_top_create_device(pmi,
                                                         get_device_index(pmi->
                                                                          memory,
                                                                          value),
                                                         false);
 
-                        if (code < 0)
-                            return code;
                         /* check for icc settings */
                     } else
                         if (!strncmp
                             (arg, "DefaultGrayProfile",
                              strlen("DefaultGrayProfile"))) {
-                        pmi->pdefault_gray_icc = arg_heap_copy(value);
+                        pmi->pdefault_gray_icc = arg_copy(value, pmi->memory);
                     } else
                         if (!strncmp
                             (arg, "DefaultRGBProfile",
                              strlen("DefaultRGBProfile"))) {
-                        pmi->pdefault_rgb_icc = arg_heap_copy(value);
+                        pmi->pdefault_rgb_icc = arg_copy(value, pmi->memory);
                     } else
                         if (!strncmp
                             (arg, "DefaultCMYKProfile",
                              strlen("DefaultCMYKProfile"))) {
-                        pmi->pdefault_cmyk_icc = arg_heap_copy(value);
+                        pmi->pdefault_cmyk_icc = arg_copy(value, pmi->memory);
                     } else
                         if (!strncmp
                             (arg, "ICCProfileDir", strlen("ICCProfileDir"))) {
-                        pmi->piccdir = arg_heap_copy(value);
+                        pmi->piccdir = arg_copy(value, pmi->memory);
                     } else {
                         char buffer[128];
 
@@ -1245,6 +1248,8 @@ pl_main_process_options(pl_main_instance_t * pmi, arg_list * pal,
                 set_debug_flags(arg, gs_debug);
                 break;
         }
+        if (code < 0)
+            return code;
     }
   out:if (help) {
         arg_finit(pal);
